@@ -45,6 +45,8 @@
 #include <assert.h>
 #include <sstream>
 
+
+
 #include <apt-pkg/pkgrecords.h>
 #include <apt-pkg/depcache.h>
 #include <apt-pkg/srcrecords.h>
@@ -112,7 +114,6 @@ const char *RPackage::srcPackage()
 {
    pkgCache::VerIterator ver = _package->VersionList();
    pkgRecords::Parser &rec=_records->Lookup(ver.FileList());
-   
    const char *s=rec.SourcePkg().empty()?name():rec.SourcePkg().c_str();
  
    return s;
@@ -380,7 +381,7 @@ bool RPackage::nextWDeps(const char *&type, const char *&what,
 }
 #endif
 
-vector<RPackage::DepInformation> RPackage::enumRDeps()
+vector<DepInformation> RPackage::enumRDeps()
 {
    vector<DepInformation> deps;
    DepInformation dep;
@@ -389,7 +390,7 @@ vector<RPackage::DepInformation> RPackage::enumRDeps()
    for(pkgCache::DepIterator D = _package->RevDependsList(); D.end() != true; D++) {
       // clear old values
       dep.isOr=dep.isVirtual=false;
-      dep.name=dep.version=dep.versionComp=dep.typeStr=NULL;
+      dep.name=dep.version=dep.versionComp=NULL;
 
       // check target and or-depends status
       pkgCache::PkgIterator Trg = D.TargetPkg();
@@ -398,7 +399,9 @@ vector<RPackage::DepInformation> RPackage::enumRDeps()
 	 dep.versionComp = "";
       }
 
-      dep.typeStr=_("Dependency of");
+      //FIXME: HACK ALARM, we need a "RDepends" type, so we use the last
+      // one in pkg-dep
+      dep.type = (pkgCache::Dep::DepType)(pkgCache::Dep::Obsoletes+1);
       dep.name = D.ParentPkg().Name();
 
       if(Trg->VersionList == 0)
@@ -541,10 +544,14 @@ string RPackage::showWhyInstBroken()
 	       requiredVersion = "("+string(Start.CompType())+string(Start.TargetVer())+")";
             if (Ver.end() == false) {
                if (FirstOr == false)
+		  // TRANSLATORS: dependency error message, example:
+		  // "apt 0.5.4 but 0.5.3 is to be installed"
                   ioprintf(out, _("\t%s %s but %s is to be installed"),
                            Start.TargetPkg().Name(), requiredVersion.c_str(),
 			   Ver.VerStr());
                else
+		  // TRANSLATORS: dependency error message, example:
+		  // "Depends: apt 0.5.4 but 0.5.3 is to be installed"
                   ioprintf(out, _(" %s: %s %s but %s is to be installed"),
                            End.DepType(), Start.TargetPkg().Name(),
 			   requiredVersion.c_str(), Ver.VerStr());
@@ -553,23 +560,35 @@ string RPackage::showWhyInstBroken()
                    true) {
                   if (Targ->ProvidesList == 0)
                      if (FirstOr == false)
+			// TRANSLATORS: dependency error message, example:
+			// "apt 0.5.4 but it is not installable"
                         ioprintf(out, _("\t%s %s but it is not installable"),
                                  Start.TargetPkg().Name(), 
 				 requiredVersion.c_str());
                      else
+			// TRANSLATORS: dependency error message, example:
+			// "Depends: apt 0.5.4  but it is not installable",
                         ioprintf(out, "%s: %s %s but it is not installable",
                                  End.DepType(), Start.TargetPkg().Name(),
 				 requiredVersion.c_str());
                   else if (FirstOr == false)
+		     // TRANSLATORS: dependency error message, example:
+		     // "apt but it is a virtual package"
                      ioprintf(out, _("\t%s but it is a virtual package"),
                               Start.TargetPkg().Name());
                   else
+		     // TRANSLATORS: dependency error message, example:
+		     // "Depends: apt but it is a virtual package"
                      ioprintf(out, _("%s: %s but it is a virtual package"),
                               End.DepType(), Start.TargetPkg().Name());
                } else if (FirstOr == false)
+		  // TRANSLATORS: dependency error message, example:
+		  // "apt but it is not going to be installed"
                   ioprintf(out, _("\t%s but it is not going to be installed"),
                            Start.TargetPkg().Name());
                else
+		  // TRANSLATORS: dependency error message, example:
+		  // "Depends: apt but it is not going to be installed"
                   ioprintf(out, _("%s: %s but it is not going to be installed"),
                            End.DepType(), Start.TargetPkg().Name());
             }
@@ -601,7 +620,7 @@ string RPackage::showWhyInstBroken()
 }
 
 
-vector<RPackage::DepInformation> RPackage::enumDeps(bool useCanidateVersion)
+vector<DepInformation> RPackage::enumDeps(bool useCanidateVersion)
 {
    vector<DepInformation> deps;
    DepInformation dep;
@@ -620,7 +639,7 @@ vector<RPackage::DepInformation> RPackage::enumDeps(bool useCanidateVersion)
 
       // clear old values
       dep.isOr=dep.isVirtual=dep.isSatisfied=false;
-      dep.name=dep.version=dep.versionComp=dep.typeStr=NULL;
+      dep.name=dep.version=dep.versionComp=NULL;
 
       // check target and or-depends status
       pkgCache::PkgIterator Trg = D.TargetPkg();
@@ -629,21 +648,18 @@ vector<RPackage::DepInformation> RPackage::enumDeps(bool useCanidateVersion)
 
       // common information
       dep.type = (pkgCache::Dep::DepType)D->Type;
-      dep.typeStr = D.DepType();
       dep.name = Trg.Name();
 
       // satisfied
       if (((*_depcache)[D] & pkgDepCache::DepGInstall) ==
           pkgDepCache::DepGInstall)
          dep.isSatisfied = true;
-
       if (Trg->VersionList == 0) {
 	 dep.isVirtual = true;
       } else {
 	 dep.version=D.TargetVer();
 	 dep.versionComp=D.CompType();
       }
-
       deps.push_back(dep);
    }
 
@@ -659,144 +675,40 @@ bool RPackage::dependsOn(const char *pkgname)
    return false;
 }
 
-
-
-#if 0
-bool RPackage::enumDeps(const char *&type, const char *&what,
-                        const char *&pkg, const char *&which, char *&summary,
-                        bool &satisfied)
+#ifdef WITH_APT_AUTH
+bool RPackage::isTrusted()
 {
-   pkgCache::VerIterator ver;
-   pkgDepCache::StateCache & state = (*_depcache)[*_package];
-
-   if (state.Keep() || state.Held()) {
-      ver = (*_depcache)[*_package].InstVerIter(*_depcache);
-
-      if (ver.end())
-         ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
-   } else {
-      //ver = _package->VersionList();
-      ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
-   }
-   if (ver.end())
+   pkgCache::VerIterator Ver;
+   pkgDepCache::StateCache & State = (*_depcache)[*_package];
+   Ver = State.CandidateVerIter(*_depcache);
+   if (Ver == 0) {
+      cerr << "CanidateVer == 0" << endl;
       return false;
-
-   _depI = ver.DependsList();
-   // uninitialized but doesn't matter, they just have to be equal    
-   _depStart = _depEnd;
-
-   return nextDeps(type, what, pkg, which, summary, satisfied);
-}
-
-
-bool RPackage::nextDeps(const char *&type, const char *&what,
-                        const char *&pkg, const char *&which, char *&summary,
-                        bool &satisfied)
-{
-   static char buffer[32];      // type
-   static char buffer2[32];     // which
-   static char buffer3[64];     // summary
-
-   while (1) {
-      if (_depStart == _depEnd) {
-         if (_depI.end())
-            return false;
-
-         _depI.GlobOr(_depStart, _depEnd);
-
-         snprintf(buffer, sizeof(buffer), "%s", _depEnd.DepType());
-      } else {
-         _depStart++;
-
-         snprintf(buffer, sizeof(buffer), "| %s", _depEnd.DepType());
-      }
-
-      if (isWeakDep(_depEnd))
-         continue;
-
-      satisfied = false;
-      if (((*_depcache)[_depStart] & pkgDepCache::DepGInstall) ==
-          pkgDepCache::DepGInstall)
-         satisfied = true;
-
-      type = buffer;
-      //cout << "type: " << buffer << endl;
-
-      pkgCache::PkgIterator depPkg = _depStart.TargetPkg();
-      what = depPkg.Name();
-
-      //cout << "what (before): " << what << endl;
-
-      if (depPkg->VersionList != 0)
-         pkg = what;
-      else if (depPkg->ProvidesList != 0)
-         pkg = depPkg.ProvidesList().OwnerPkg().Name();
-      else
-         pkg = 0;
-      //cout << "what (after): " << what << endl;     
-
-      if (_depStart.TargetVer()) {
-         snprintf(buffer2, sizeof(buffer2), "(%s %s)",
-                  _depStart.CompType(), _depStart.TargetVer());
-         which = buffer2;
-      } else {
-         which = "";
-      }
-      //cout << "which: " << buffer2 << endl;
-
-      buffer3[0] = 0;
-      if (!satisfied && depPkg->ProvidesList == 0) {
-         pkgCache::VerIterator Ver =
-            (*_depcache)[depPkg].InstVerIter(*_depcache);
-
-         if (!Ver.end())
-            snprintf(buffer3, sizeof(buffer3), _("%s is/will be installed"),
-                     Ver.VerStr());
-         else {
-            if ((*_depcache)[depPkg].CandidateVerIter(*_depcache).end()) {
-               if (depPkg->ProvidesList == 0)
-                  strcpy(buffer3, _("package is not installable"));
-               else
-                  strcpy(buffer3, _("package is a virtual package"));
-            } else
-               strcpy(buffer3, _("package is/will not be installed"));
-         }
-      } else if (satisfied) {
-         strcpy(buffer3, _("dependency is satisfied"));
-      }
-      summary = buffer3;
-      //cout << "summary: " << summary << endl;
-
-      break;
    }
-   return true;
-}
-
-RPackage::UpdateImportance RPackage::updateImportance()
-{
-   return IUnknown;             //(*name()) == 'a' ? ISecurity : INormal;
-}
-
-
-const char *RPackage::updateSummary()
-{
-   return "Test\nAdvisory\nYour computer will blow if you don't update.";
-   return NULL;
-}
-
-
-const char *RPackage::updateDate()
-{
-   return "2001-1-1 12:43 GMT";
-   return NULL;
-}
-
-
-const char *RPackage::updateURL()
-{
-   return "http://sekure.org/~dumped/exploitz";
-   return NULL;
+   pkgSourceList *Sources=_lister->getCache()->list();
+   for (pkgCache::VerFileIterator i = Ver.FileList(); i.end() == false; i++)
+   {
+      pkgIndexFile *Index;
+      if (Sources->FindIndex(i.File(),Index) == false)
+         continue;
+      if (_config->FindB("Debug::pkgAcquire::Auth", false))
+      {
+         std::cerr << "Checking index: " << Index->Describe()
+                   << "(Trusted=" << Index->IsTrusted() << ")\n";
+      }
+      if (Index->IsTrusted())
+         return true;
+   }
+   
+   return false;
 }
+#else
+// with apt-authentication we always trust that the package come from
+// a trusted source
+bool RPackage::isTrusted() 
+{ 
+   return true; 
+};
 #endif
 
 bool RPackage::wouldBreak()
@@ -826,6 +738,7 @@ void RPackage::setInstall()
    _depcache->MarkInstall(*_package, true);
    pkgDepCache::StateCache & State = (*_depcache)[*_package];
 
+   // FIXME: can't we get rid of it here?
    // if there is something wrong, try to fix it
    if (!State.Install() || _depcache->BrokenCount() > 0) {
       pkgProblemResolver Fix(_depcache);
@@ -834,6 +747,7 @@ void RPackage::setInstall()
       Fix.Resolve(true);
    }
 
+
 #ifdef WITH_LUA
    _lua->SetDepCache(_depcache);
    _lua->SetGlobal("package", ((pkgCache::Package *) * _package));
@@ -849,8 +763,6 @@ void RPackage::setInstall()
 void RPackage::setReInstall(bool flag)
 {
     _depcache->SetReInstall(*_package, flag);
-    _config->Set("APT::Get::ReInstall", flag);
-    
     if (_notify)
 	_lister->notifyChange(this);
 }
@@ -867,32 +779,13 @@ void RPackage::setRemove(bool purge)
    Fix.InstallProtect();
    Fix.Resolve(true);
 
+   _depcache->SetReInstall(*_package, false);
    _depcache->MarkDelete(*_package, purge);
 
    if (_notify)
       _lister->notifyChange(this);
 }
 
-void create_tmpfile(const char *pattern, char **filename, FILE **out)
-{
-   char *tmpdir = NULL;
-
-   // remove from pin-file
-   if (!((tmpdir = getenv("TMPDIR")))) {
-      tmpdir = getenv("TMP");
-   }
-   if (!tmpdir) {
-      tmpdir = "/tmp";
-   }
-   *filename = (char *)malloc(strlen(tmpdir) + strlen(pattern) + 2);
-   assert(filename);
-
-   strcpy(*filename, tmpdir);
-   strcat(*filename, "/");
-   strcat(*filename, pattern);
-   int tmp_fd = mkstemp(*filename);
-   *out = fdopen(tmp_fd, "w+b");
-}
 
 string RPackage::getChangelogFile(pkgAcquire *fetcher)
 {
@@ -911,11 +804,14 @@ string RPackage::getChangelogFile(pkgAcquire *fetcher)
    if(srcpkg.size()>3 && srcpkg[0]=='l' && srcpkg[1]=='i' && srcpkg[2]=='b')
       prefix=std::string("lib")+srcpkg[3];
 
-   string verstr = availableVersion();
+   string verstr;
+   if(availableVersion() != NULL) 
+      verstr = availableVersion();
+   
    if(verstr.find(':')!=verstr.npos)
       verstr=string(verstr, verstr.find(':')+1);
    char uri[512];
-   snprintf(uri,512,"http://packages.debian.org/changelogs/pool/%s/%s/%s/%s_%s/changelog",
+   snprintf(uri,512,"http://changelogs.ubuntu.com/changelogs/pool/%s/%s/%s/%s_%s/changelog",
                                src_section.c_str(),
                                prefix.c_str(),
                                srcpkg.c_str(),
@@ -925,7 +821,7 @@ string RPackage::getChangelogFile(pkgAcquire *fetcher)
    //cout << "uri is: " << uri << endl;
 
    // no need to translate this, the changelog is in english anyway
-   string filename = RConfDir()+"/tmp_cl";
+   string filename = RTmpDir()+"/tmp_cl";
    ofstream out(filename.c_str());
    out << "Failed to fetch the changelog for " << name() << endl;
    out << "URI was: " << uri << endl;
@@ -953,11 +849,9 @@ string RPackage::getCanidateOrigin()
 void RPackage::setPinned(bool flag)
 {
    FILE *out;
-   char pattern[] = "syn-XXXXXX";
-   char *filename = NULL;
    struct stat stat_buf;
 
-   string File =RConfDir() + "/preferences";
+   string File =RStateDir() + "/preferences";
 
    _boolFlags = flag ? (_boolFlags | FPinned) : (_boolFlags & FPinned);
 
@@ -981,7 +875,9 @@ void RPackage::setPinned(bool flag)
    } else {
       // delete package from pinning file
       stat(File.c_str(), &stat_buf);
-      create_tmpfile(pattern, &filename, &out);
+      // create a tmp_pin file in the internal dir
+      string filename = RTmpDir()+"/tmp_pin";
+      FILE *out = fopen(filename.c_str(),"w");
       if (out == NULL)
          cerr << "error opening tmpfile: " << filename << endl;
       FileFd Fd(File, FileFd::ReadOnly);
@@ -1007,9 +903,9 @@ void RPackage::setPinned(bool flag)
          }
       }
       fflush(out);
-      rename(filename, File.c_str());
+      rename(filename.c_str(), File.c_str());
       chmod(File.c_str(), stat_buf.st_mode);
-      free(filename);
+      fclose(out);
    }
 }
 
@@ -1304,6 +1200,19 @@ static char *parseDescription(string descr)
 string RPackage::component()
 {
    string res;
+#ifdef WITH_APT_AUTH
+   // the apt-secure patch breaks File.Component
+   const char *s = _package->Section();
+   if(s == NULL)
+      return "";
+
+   string src_section(s);
+   if(src_section.find('/')!=src_section.npos)
+      src_section=string(src_section, 0, src_section.find('/'));
+   else
+      src_section="main";
+   res = src_section;
+#else
    pkgCache::VerIterator Ver;
    pkgDepCache::StateCache & State = (*_depcache)[*_package];
    if (State.CandidateVer == 0) {
@@ -1320,7 +1229,7 @@ string RPackage::component()
    }
 
    res = File.Component();
-
+#endif
    return res;
 }
 
@@ -1385,7 +1294,7 @@ void RPackageStatus::init()
       markUnsupported = true;
 
       // read supported labels
-      labels = _config->Find("Synaptic::supported-label", "Debian Debian-Security");
+      labels = _config->Find("Synaptic::supported-label", "Ubuntu");
       stringstream sst1(labels);
       while(!sst1.eof()) {
 	 sst1 >> s;
@@ -1393,7 +1302,7 @@ void RPackageStatus::init()
       }
       
       // read supported components
-      components = _config->Find("Synaptic::supported-components", "main updates/main");
+      components = _config->Find("Synaptic::supported-components", "main updates/main restricted");
       stringstream sst2(components);
       while(!sst2.eof()) {
 	 sst2 >> s;
