@@ -70,11 +70,53 @@ void RGSummaryWindow::buildTree(RGSummaryWindow *me)
    vector<RPackage *> toReInstall;
    vector<RPackage *> toUpgrade;
    vector<RPackage *> toRemove;
+   vector<RPackage *> toPurge;
    vector<RPackage *> toDowngrade;
+#ifdef WITH_APT_AUTH
+   vector<string> notAuthenticated;
+#endif
    double sizeChange;
 
    lister->getDetailedSummary(held, kept, essential, toInstall, toReInstall,
-			      toUpgrade,  toRemove, toDowngrade, sizeChange);
+			      toUpgrade,  toRemove, toPurge, toDowngrade, 
+#ifdef WITH_APT_AUTH
+			      notAuthenticated,
+#endif
+			      sizeChange);
+
+#ifdef WITH_APT_AUTH
+   if(notAuthenticated.size() > 0) {
+      GtkWidget *label;
+      label = glade_xml_get_widget(me->_gladeXML, "label_auth_warning");
+      assert(label);
+      // FIXME: make this a message from a trust class (and remeber to
+      // change the text in rgchangeswindow then too)
+      gchar *msg = g_strdup_printf("<span weight=\"bold\" size=\"larger\">%s"
+				   "</span>\n\n%s",
+				   _("Warning"), 
+				   _("You are about to install software that "
+				     "<b>can't be authenticated</b>! Doing "
+				     "this could allow a malicious individual "
+				     "to damage or take control of your "
+				     "system."));
+      gtk_label_set_markup(GTK_LABEL(label), msg);
+      gtk_widget_show(label);
+      g_free(msg);
+
+      /* not authenticated */
+      gtk_tree_store_append(me->_treeStore, &iter, NULL);
+      gtk_tree_store_set(me->_treeStore, &iter,
+                         PKG_COLUMN, _("NOT AUTHENTICATED"), -1);
+
+      for (vector<string>::const_iterator p = notAuthenticated.begin();
+           p != notAuthenticated.end(); p++) {
+         gtk_tree_store_append(me->_treeStore, &iter_child, &iter);
+         gtk_tree_store_set(me->_treeStore, &iter_child,
+                            PKG_COLUMN, (*p).c_str(), -1);
+      }
+   }
+#endif
+
 
    if (essential.size() > 0) {
       /* (Essentail) removed */
@@ -115,6 +157,20 @@ void RGSummaryWindow::buildTree(RGSummaryWindow *me)
       }
    }
 
+   if (toPurge.size() > 0) {
+      /* removed */
+      gtk_tree_store_append(me->_treeStore, &iter, NULL);
+      gtk_tree_store_set(me->_treeStore, &iter,
+                         PKG_COLUMN, _("To be completely removed (including configuration files)"), 
+			 -1);
+      for (vector<RPackage *>::const_iterator p = toPurge.begin();
+           p != toPurge.end(); p++) {
+         gtk_tree_store_append(me->_treeStore, &iter_child, &iter);
+         gtk_tree_store_set(me->_treeStore, &iter_child,
+                            PKG_COLUMN, (*p)->name(), -1);
+      }
+   }
+
    if (toUpgrade.size() > 0) {
       gtk_tree_store_append(me->_treeStore, &iter, NULL);
       gtk_tree_store_set(me->_treeStore, &iter,
@@ -157,7 +213,7 @@ void RGSummaryWindow::buildTree(RGSummaryWindow *me)
    if (held.size() > 0) {
       gtk_tree_store_append(me->_treeStore, &iter, NULL);
       gtk_tree_store_set(me->_treeStore, &iter,
-                         PKG_COLUMN, _("To be not upgraded"), -1);
+                         PKG_COLUMN, _("Unchanged"), -1);
       for (vector<RPackage *>::const_iterator p = held.begin();
            p != held.end(); p++) {
          gtk_tree_store_append(me->_treeStore, &iter_child, &iter);
@@ -165,6 +221,9 @@ void RGSummaryWindow::buildTree(RGSummaryWindow *me)
                             PKG_COLUMN, (*p)->name(), -1);
       }
    }
+
+   
+
 }
 
 
@@ -183,12 +242,18 @@ void RGSummaryWindow::buildLabel(RGSummaryWindow *me)
    vector<RPackage *> toReInstall;
    vector<RPackage *> toUpgrade;
    vector<RPackage *> toRemove;
+   vector<RPackage *> toPurge;
    vector<RPackage *> toDowngrade;
+   vector<string> notAuthenticated;
    double sizeChange;
 
    me->_lister->getDetailedSummary(held, kept, essential,
 				   toInstall,toReInstall,toUpgrade, 
-				   toRemove, toDowngrade, sizeChange);
+				   toRemove, toPurge, toDowngrade, 
+#ifdef WITH_APT_AUTH
+				   notAuthenticated,
+#endif
+				   sizeChange);
 
    for (vector<RPackage *>::const_iterator p = essential.begin();
         p != essential.end(); p++) {
@@ -208,6 +273,14 @@ void RGSummaryWindow::buildLabel(RGSummaryWindow *me)
       g_free(str);
    }
 
+   for (vector<RPackage *>::const_iterator p = toPurge.begin();
+        p != toPurge.end(); p++) {
+      str = g_strdup_printf(_("<b>%s</b> will be removed with configuration\n"), 
+			    (*p)->name());
+      text += str;
+      g_free(str);
+   }
+
    for (vector<RPackage *>::const_iterator p = toRemove.begin();
         p != toRemove.end(); p++) {
       str = g_strdup_printf(_("<b>%s</b> will be removed\n"), (*p)->name());
@@ -330,7 +403,7 @@ RGSummaryWindow::RGSummaryWindow(RGWindow *wwin, RPackageLister *lister)
                       (GtkSignalFunc) clickedCancel, this);
 
    int toInstall, toReInstall, toRemove, toUpgrade, toDowngrade;
-   int held, kept, essential;
+   int held, kept, essential, unAuthenticated;
    double sizeChange, dlSize;
    int dlCount;
    GString *msg = g_string_new("");
@@ -338,7 +411,7 @@ RGSummaryWindow::RGSummaryWindow(RGWindow *wwin, RPackageLister *lister)
 
    lister->getSummary(held, kept, essential,
                       toInstall, toReInstall, toUpgrade, toRemove, 
-		      toDowngrade, sizeChange);
+		      toDowngrade, unAuthenticated,sizeChange);
    lister->getDownloadSummary(dlCount, dlSize);
 
 #if 0
@@ -422,7 +495,10 @@ RGSummaryWindow::RGSummaryWindow(RGWindow *wwin, RPackageLister *lister)
    gtk_label_set_markup(GTK_LABEL(_summarySpaceL), msg_space->str);
    g_string_free(msg, TRUE);
    g_string_free(msg_space, TRUE);
-   skipTaskbar(true);
+   if(!_config->FindB("Volatile::HideMainwindow", false))
+      skipTaskbar(true);
+   else
+      skipTaskbar(false);
 }
 
 
@@ -430,7 +506,7 @@ RGSummaryWindow::RGSummaryWindow(RGWindow *wwin, RPackageLister *lister)
 bool RGSummaryWindow::showAndConfirm()
 {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_dlonlyB),
-                                _config->FindB("Synaptic::Download-Only",
+                                _config->FindB("Volatile::Download-Only",
                                                false));
 
    show();
@@ -444,7 +520,7 @@ bool RGSummaryWindow::showAndConfirm()
                              , false) == false)
       return false;
 
-   _config->Set("Synaptic::Download-Only",
+   _config->Set("Volatile::Download-Only",
                 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_dlonlyB)) ?
                 "true" : "false");
 #ifdef HAVE_RPM
